/**
 * @author pdhindsa
 */
jQuery.paramquery.pqGrid.regional['en'] = {
    strAdd: "Add",
    strDelete: "Delete",        
    strEdit: "Edit",
    strLoading: "Loading",
    strNextResult: "Next Result",        
    strNoRows: "No rows to display.",        
    strNothingFound: "Nothing found",
    strPrevResult: "Previous Result",
    strSearch: "Search",
    strSelectedmatches: "Selected {0} of {1} match(es)"    
};
jQuery.paramquery.pqPager.regional['en']={
    strDisplay:"Displaying {0} to {1} of {2} items.",		
    strFirstPage:"First Page",
    strLastPage:"Last Page",
    strNextPage:"Next Page",        
    strPage:"Page {0} of {1}",
    strPrevPage:"Previous Page",	
    strRefresh:"Refresh",	
    strRpp:"Records per page: {0}"	
};