/**
 * @author petigrafix
 */
$.paramquery.pqGrid.regional['de'] = { 
    strLoading: "Laden",
    strAdd: "Hinzufügen",
    strEdit: "Bearbeiten",
    strDelete: "Löschen",
    strSearch: "Suchen",
    strNothingFound: "Kein Ergebnis gefunden",
    strSelectedmatches:"Ergebnis {0} von {1}",
    strPrevResult: "Vorheriges Ergebnis",
    strNextResult: "Weiter Ergebnis",
    strNoRows: "Keine anzuzeigenden Zeilen."      
}; 
$.paramquery.pqPager.regional['de'] = { 
    strPage:"Seite {0} von {1}",
    strFirstPage:"Erste Seite",
    strPrevPage:"Vorherige Seite",
    strNextPage:"Nächste Seite",
    strLastPage:"Letzte Seite",
    strRefresh:"Neu laden",	
    strRpp:"Ergebnisse pro Seite: {0}",
    strDisplay:"Ergebnis {0} bis {1} von {2}."
};